@@ -1,14 +1,11 @@
-__version__ = "1.20.12"
+__version__ = "1.24.1"
 
 from .environment import Environment as Environment
 from .helpers.ArchiveStorageManager import (
     set_assetbundle_decrypt_key as set_assetbundle_decrypt_key,
 )
 
-
-def load(*args, fs=None, **kwargs):
-    return Environment(*args, fs=fs, **kwargs)
-
+load = Environment
 
 # backward compatibility
 AssetsManager = Environment
